from sqlalchemy import *

from dataProviderBase import *
from T0Mon.utils import *
from T0Mon.consts import *
from T0Mon.sqlManager import *

from T0Mon.globals import logger
import T0Mon.config

### Module for classes providing data about T0 files ###

class BaseDP(PagedTableDataProvider):
    cols = Cols([
        Col("Id", decorator = buildRepackLink),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
#        Col("Status"),            # TODO : check these columns in the database, add to the final version of this table
#        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, type, orderby = 0, asc = True, page = 0):
	self.type = type
        PagedTableDataProvider.__init__(self, orderby, asc, page)

    def _buildCountQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)

        self.countQuery = """ SELECT COUNT(DISTINCT f.id)
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
                                         """ + self.fromCond + """
                                WHERE dt.name = '""" + self.type + """' """ + self.whereCond + """ """

    def _buildMainQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
                                        INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id)
                                         """ + self.fromCond + """
                                WHERE dt.name = '""" + self.type + """' """ + self.whereCond + """
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class BaseRepackDP(BaseDP):

    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseDP.__init__(self, 'RAW', orderby, asc, page)
        self.title = "Repacked files"


class RepackByIdsDP(BaseRepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)

        repackedTable = Tbl(TABLES.REPACKED)

        self.whereCond = repackedTable.c.repacked_id.in_(ids)

        BaseRepackDP._load(self)

class RepackByRunIdDP(BaseRepackDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Repacked files"

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
	wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
	wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

	self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Mergeable' """
	
	self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid) 
			    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
			    INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRepackDP._load(self)

class RepackByRepackMergeIdsDP(BaseRepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)

        repackedTable = Tbl(TABLES.REPACKED)
        parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)

        self.whereCond = \
            or_(
                repackedTable.c.repacked_id.in_(select([parentageTable.c.input_id], parentageTable.c.output_id.in_(ids))),
                and_(
                    repackedTable.c.repacked_id.in_(ids),
                    not_(repackedTable.c.repacked_id.in_(select([parentageTable.c.output_id])))
                )
            )
        BaseRepackDP._load(self)

class RepackByRepackJobIdsDP(BaseRepackDP):    # TODO -> Repasar esta funcion
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)

	repackStreamerAssocTable = Tbl(TABLES.REPACK_STREAMER_ASSOC, True)
	jobDatasetStreamerAssocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC, True)
	wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)

	ids_str = ""
	for id in ids:
		ids_str += str(id)

        self.whereCond = """ AND f.id IN (SELECT repacked_id 
						FROM """ + repackStreamerAssocTable + """ 
						WHERE streamer_id IN (SELECT streamer_id 
									FROM """ + jobDatasetStreamerAssocTable + """ 
									WHERE job_id IN (""" + ids_str + """))
						) 
			     AND dp.primary_dataset IN (SELECT primary_dataset_id
							FROM """ + jobDatasetStreamerAssocTable + """
                                                        WHERE job_id IN (""" + ids_str + """))"""
	
	self.fromCond = """ """

        BaseRepackDP._load(self)

class BaseRepackMergeDP(BaseRepackDP):
    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
        self.title = "Repacked files"


class RepackMergeByRecoJobIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        assocTable = Tbl(TABLES.RECO_JOB_REPACK_ASSOC)
        self.whereCond = repackedTable.c.repacked_id.in_(
            select([assocTable.c.repacked_id], assocTable.c.job_id.in_(ids))
        )
        BaseRepackMergeDP._load(self)

class RepackMergeByIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        self.whereCond = repackedTable.c.repacked_id.in_(ids)
        BaseRepackMergeDP._load(self)

class RepackMergeByRepackMergeJobIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)
        self.whereCond = repackedTable.c.repacked_id.in_(
            select([parentageTable.c.output_id], parentageTable.c.input_id.in_(
                select([assocTable.c.repacked_id], assocTable.c.job_id.in_(ids))
            ))
        )
        BaseRepackMergeDP._load(self)

class RepackMergeByRepackIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)
        self.whereCond = \
            and_(
                or_(
                    repackedTable.c.status == REPACK_RECONSTRUCTION_QUEUED,
                    repackedTable.c.status == REPACK_RECONSTRUCTION_DISABLED
                ),
                or_(
                    repackedTable.c.repacked_id.in_(
                        select([parentageTable.c.output_id],
                            parentageTable.c.input_id.in_(ids)
                        )
                    ),
                    repackedTable.c.repacked_id.in_(ids)
                )
            )
        if len(ids) != 0:
            BaseRepackMergeDP._load(self)

class RepackMergeByRunIdDP(BaseRepackMergeDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Exportable' """

        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
	                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
            	            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRepackMergeDP._load(self)

class BaseRecoDP(BaseDP):
    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseDP.__init__(self, 'RECO', orderby, asc, page)
        self.title = "Reco files"

class RecoMergeByRunIdDP(BaseRecoDP):
        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseRecoDP.__init__(self, orderby, asc, page)
		self.title = "Reco files"

                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
                wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

                self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Exportable' """

                self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
		                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
			            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

                BaseRecoDP._load(self)


class BaseAlcaskimDP(BaseDP):
    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseDP.__init__(self, 'ALCARECO', orderby, asc, page)
        self.title = "AlcaSkim files"

class AlcaskimByRunIdDP(BaseAlcaskimDP):
        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
	        self.title = "Temporary AlcaSkim files"

	        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        	wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
	        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        	self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Mergeable' """

	        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
        	                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                	            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

                BaseAlcaskimDP._load(self)


class AlcaskimMergeByRunIdDP(BaseAlcaskimDP):
        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
                self.title = "AlcaSkim files"

                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
                wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

                self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Exportable' """

                self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                                    INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

                BaseAlcaskimDP._load(self)

class RecoByIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        recoTable = Tbl(TABLES.RECONSTRUCTED)
        self.whereCond = recoTable.c.reconstructed_id.in_(ids) 
        BaseRecoDP._load(self)

class RecoByRunIdDP(BaseRecoDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Reco files"

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Mergeable' """

        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                            INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRecoDP._load(self)

class RecoByRecoJobIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        recoTable = Tbl(TABLES.RECONSTRUCTED)
        assocTable = Tbl(TABLES.RECO_JOB_REPACK_ASSOC)
        parentageTable = Tbl(TABLES.REPACKED_RECO_PARENTAGE)
        self.whereCond = recoTable.c.reconstructed_id.in_(
            select([parentageTable.c.output_id], parentageTable.c.input_id.in_(
                select([assocTable.c.repacked_id], assocTable.c.job_id.in_(ids))
            ))
        )
        BaseRecoDP._load(self)

class StreamerDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id", decorator = buildStreamerLink),
        Col("Lumi Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
        Col("Name"),
        Col("E"),   # TODO : check if this column is still necessary, it doesn't appear in streamer table
#        Col("S"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.title = 'Streamer files'

    def _buildCountQuery(self):
        streamerTable = Tbl(TABLES.STREAMER)

        self.countQuery = select(
            [func.count(streamerTable.c.streamer_id)],
            self.whereCond
        )

    def _buildMainQuery(self):
        streamerTable = Tbl(TABLES.STREAMER)
        streamTable = Tbl(TABLES.STREAM)

        c = (
            streamerTable.c.streamer_id,
            streamerTable.c.lumi_id,
            streamerTable.c.filesize,
            streamerTable.c.events,
            streamTable.c.name,
            streamerTable.c.exportable,
#            streamerTable.c.splitable,
            streamerTable.c.lfn
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
	    from_obj = [streamerTable.outerjoin(streamTable, streamerTable.c.stream_id == streamTable.c.id)]
        ).order_by(o)

class StreamerByRunIdDP(StreamerDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        self.whereCond = (streamerTable.c.run_id == runid)
        StreamerDP._load(self)

class StreamerByIdsDP(StreamerDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        self.whereCond = (streamerTable.c.streamer_id.in_(ids))
        StreamerDP._load(self)

class StreamerByRepackJobIdsDP(StreamerDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        assocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        self.whereCond = streamerTable.c.streamer_id.in_(
            select([func.distinct(assocTable.c.streamer_id)], assocTable.c.job_id.in_(ids))
        )
        StreamerDP._load(self)



